@@ -56,10 +56,13 @@
 void zxid_sha1_file(zxid_conf* cf, char* name, char* sha1)
 {
   int gotall;
-  char buf[4096];
-  memset(sha1, 0, 20);
-  gotall = read_all(sizeof(buf), buf, "sha1_file", "%s%s", cf->path, name);
+  char* buf;
+  ZERO(sha1, 20);
+  buf = read_all_alloc(cf->ctx, "sha1_file", 1, &gotall, "%s%s", cf->path, name);
+  if (!buf)
+    return;
   SHA1(buf, gotall, sha1);
+  ZX_FREE(cf->ctx, buf);
 }
 #endif
 
@@ -132,7 +135,7 @@ RSA* zxid_extract_private_key(char* buf, char* name)
 X509* zxid_read_cert(zxid_conf* cf, char* name)
 {
   char buf[4096];
-  int got = read_all(sizeof(buf), buf, "read_cert", "%s" ZXID_PEM_DIR "%s", cf->path, name);
+  int got = read_all(sizeof(buf), buf, "read_cert", 1, "%s" ZXID_PEM_DIR "%s", cf->path, name);
   if (!got && cf->auto_cert)
      zxid_mk_self_sig_cert(cf, sizeof(buf), buf, "read_cert", name);
   return zxid_extract_cert(buf, name);
@@ -144,7 +147,7 @@ X509* zxid_read_cert(zxid_conf* cf, char* name)
 RSA* zxid_read_private_key(zxid_conf* cf, char* name)
 {
   char buf[4096];
-  int got = read_all(sizeof(buf), buf, "read_private_key", "%s" ZXID_PEM_DIR "%s", cf->path, name);
+  int got = read_all(sizeof(buf), buf, "read_private_key", 1, "%s" ZXID_PEM_DIR "%s", cf->path, name);
   if (!got && cf->auto_cert)
      zxid_mk_self_sig_cert(cf, sizeof(buf), buf, "read_private_key", name);
   return zxid_extract_private_key(buf, name);
@@ -228,7 +231,8 @@ int zxid_ent_cache_mx_init = 0;
  *
  * cf:: Pointer to previously allocated configuration object
  * path:: Since this configuration option is so fundamental, it can
- *     be supplied directly as argument.
+ *     be supplied directly as argument. However, unlike zxid_new_conf()
+ *     this does not cause the config file to be read.
  * return:: 0 on success (currently, 2008, this function can not
  *     fail - thus it is common to ignore the return value)
  *
@@ -240,7 +244,7 @@ int zxid_ent_cache_mx_init = 0;
  */
 
 /* Called by:  zxid_conf_to_cf_len, zxid_init_conf_ctx */
-int zxid_init_conf(zxid_conf* cf, char* zxid_path)
+int zxid_init_conf(zxid_conf* cf, const char* zxid_path)
 {
   DD("Initconf with path(%s)", zxid_path);
   cf->magic = ZXID_CONF_MAGIC;
@@ -276,8 +280,11 @@ int zxid_init_conf(zxid_conf* cf, char* zxid_path)
   cf->wspcgicmd    = ZXID_WSPCGICMD;
   cf->nameid_enc   = ZXID_NAMEID_ENC;
   cf->post_a7n_enc = ZXID_POST_A7N_ENC;
+  cf->canon_inopt  = ZXID_CANON_INOPT;
+  if (cf->ctx) cf->ctx->canon_inopt = cf->canon_inopt;
   cf->enckey_opt   = ZXID_ENCKEY_OPT;
   cf->idpatopt     = ZXID_IDPATOPT;
+  cf->idp_list_meth   = ZXID_IDP_LIST_METH;
   cf->di_allow_create = ZXID_DI_ALLOW_CREATE;
   cf->di_nid_fmt   = ZXID_DI_NID_FMT;
   cf->di_a7n_enc   = ZXID_DI_A7N_ENC;
@@ -309,6 +316,7 @@ int zxid_init_conf(zxid_conf* cf, char* zxid_path)
   cf->ses_cookie_name   = ZXID_SES_COOKIE_NAME;
   cf->user_local        = ZXID_USER_LOCAL;
   cf->idp_ena           = ZXID_IDP_ENA;
+  cf->imps_ena          = ZXID_IMPS_ENA;
   cf->as_ena            = ZXID_AS_ENA;
   cf->pdp_ena           = ZXID_PDP_ENA;
 
@@ -377,6 +385,9 @@ int zxid_init_conf(zxid_conf* cf, char* zxid_path)
   cf->bare_url_entityid = ZXID_BARE_URL_ENTITYID;
   cf->show_tech         = ZXID_SHOW_TECH;
   cf->idp_sel_page      = ZXID_IDP_SEL_PAGE;
+  cf->idp_sel_templ_file= ZXID_IDP_SEL_TEMPL_FILE;
+  cf->idp_sel_templ     = ZXID_IDP_SEL_TEMPL;
+#if 0
   cf->idp_sel_start     = ZXID_IDP_SEL_START;
   cf->idp_sel_new_idp   = ZXID_IDP_SEL_NEW_IDP;
   cf->idp_sel_our_eid   = ZXID_IDP_SEL_OUR_EID;
@@ -384,11 +395,19 @@ int zxid_init_conf(zxid_conf* cf, char* zxid_path)
   cf->idp_sel_tech_site = ZXID_IDP_SEL_TECH_SITE;
   cf->idp_sel_footer    = ZXID_IDP_SEL_FOOTER;
   cf->idp_sel_end       = ZXID_IDP_SEL_END;
+#endif
 
   cf->an_page           = ZXID_AN_PAGE;
   cf->an_templ_file     = ZXID_AN_TEMPL_FILE;
   cf->an_templ          = ZXID_AN_TEMPL;
 
+  cf->post_templ_file   = ZXID_POST_TEMPL_FILE;
+  cf->post_templ        = ZXID_POST_TEMPL;
+
+  cf->err_page          = ZXID_ERR_PAGE;
+  cf->err_templ_file    = ZXID_ERR_TEMPL_FILE;
+  cf->err_templ         = ZXID_ERR_TEMPL;
+
   cf->new_user_page     = ZXID_NEW_USER_PAGE;
   cf->recover_passwd    = ZXID_RECOVER_PASSWD;
   cf->atsel_page        = ZXID_ATSEL_PAGE;
@@ -424,7 +443,7 @@ int zxid_init_conf(zxid_conf* cf, char* zxid_path)
 /* Called by:  dirconf, main x2, zx_init_ctx, zxid_az, zxid_simple_len */
 void zx_reset_ctx(struct zx_ctx* ctx)
 {
-  memset(ctx, 0, sizeof(struct zx_ctx));
+  ZERO(ctx, sizeof(struct zx_ctx));
   LOCK_INIT(ctx->mx);
   ctx->guard_seen_n.seen_n = &ctx->guard_seen_p;
   ctx->guard_seen_p.seen_p = &ctx->guard_seen_n;
@@ -450,13 +469,18 @@ struct zx_ctx* zx_init_ctx()
   return ctx;
 }
 
-/*() If zxid_path is supplied as NULL, then a minimal initialization of
+/*() Minimal initialization of
  * the context is performed. Certificate and key operations as well as
  * CURL initialization are omitted. However the zx_ctx is installed so
- * that memory allocation against the context should work. */
+ * that memory allocation against the context should work.
+ * Supplying zxid_path merely initializes the PATH config option,
+ * but does not cause configuration file to be read.
+ *
+ * Just initializes the config object to factory defaults (see zxidconf.h).
+ * Previous content of the config object is lost. */
 
 /* Called by:  zxcot_main, zxid_conf_to_cf_len, zxid_new_conf */
-zxid_conf* zxid_init_conf_ctx(zxid_conf* cf, char* zxid_path)
+zxid_conf* zxid_init_conf_ctx(zxid_conf* cf, const char* zxid_path)
 {
 #if 0
   fprintf(stderr, "Waiting 60 secs for gdb attach...\n");
@@ -478,9 +502,10 @@ zxid_conf* zxid_init_conf_ctx(zxid_conf* cf, char* zxid_path)
   return cf;
 }
 
-/*() Allocate conf object, but do not actually initialize it with default config
- * or config file.
- * See: zxid_new_conf_to_cf() for a more complete solution. */
+/*() Allocate conf object and initialize it with default config or config file.
+ * See zxid_new_conf_to_cf() for a more complete solution.
+ * Just initializes the config object to factory defaults (see zxidconf.h).
+ * Previous content of the config object is lost. */
 
 /* Called by:  main x6, test_ibm_cert_problem, test_ibm_cert_problem_enc_dec, test_mode */
 zxid_conf* zxid_new_conf(const char* zxid_path)
@@ -992,6 +1017,7 @@ scan_end:
       if (!strcmp(n, "CONTACT_EMAIL"))   { cf->contact_email = v; break; }
       if (!strcmp(n, "CONTACT_TEL"))     { cf->contact_tel = v; break; }
       if (!strcmp(n, "COUNTRY"))         { cf->country = v; break; }
+      if (!strcmp(n, "CANON_INOPT"))     { SCAN_INT(v, cf->canon_inopt); if (cf->ctx) cf->ctx->canon_inopt = cf->canon_inopt; break; }
       goto badcf;
     case 'D':  /* DUP_A7N_FATAL, DUP_MSG_FATAL */
       if (!strcmp(n, "DEFAULTQS"))       { cf->defaultqs = v; break; }
@@ -1005,6 +1031,9 @@ scan_end:
       if (!strcmp(n, "ERR"))             { SCAN_INT(v, cf->log_err); break; }
       if (!strcmp(n, "ERR_IN_ACT"))      { SCAN_INT(v, cf->log_err_in_act); break; }
       if (!strcmp(n, "ENCKEY_OPT"))      { SCAN_INT(v, cf->enckey_opt); break; }
+      if (!strcmp(n, "ERR_PAGE"))        { cf->err_page = v; break; }
+      if (!strcmp(n, "ERR_TEMPL_FILE"))  { cf->err_templ_file = v; break; }
+      if (!strcmp(n, "ERR_TEMPL"))       { cf->err_templ = v; break; }
       goto badcf;
     case 'F':
       if (!strcmp(n, "FEDUSERNAME_SUFFIX")) { cf->fedusername_suffix = v; break; }
@@ -1013,6 +1042,7 @@ scan_end:
       if (!strcmp(n, "ISSUE_A7N"))       { SCAN_INT(v, cf->log_issue_a7n); break; }
       if (!strcmp(n, "ISSUE_MSG"))       { SCAN_INT(v, cf->log_issue_msg); break; }
       if (!strcmp(n, "ISSUE_AUTHNCTX_PW")) { cf->issue_authnctx_pw = v; break; }
+#if 0
       if (!strcmp(n, "IDP_SEL_START"))   { cf->idp_sel_start = v; break; }
       if (!strcmp(n, "IDP_SEL_NEW_IDP")) { cf->idp_sel_new_idp = v; break; }
       if (!strcmp(n, "IDP_SEL_OUR_EID")) { cf->idp_sel_our_eid = v; break; }
@@ -1020,10 +1050,15 @@ scan_end:
       if (!strcmp(n, "IDP_SEL_TECH_SITE")) { cf->idp_sel_tech_site =v; break; }
       if (!strcmp(n, "IDP_SEL_FOOTER"))  { cf->idp_sel_footer = v; break; }
       if (!strcmp(n, "IDP_SEL_END"))     { cf->idp_sel_end = v; break; }
+#endif
       if (!strcmp(n, "IDP_SEL_PAGE"))    { cf->idp_sel_page = v; break; }
+      if (!strcmp(n, "IDP_SEL_TEMPL_FILE")) { cf->idp_sel_templ_file = v; break; }
+      if (!strcmp(n, "IDP_SEL_TEMPL"))   { cf->idp_sel_templ = v; break; }
       if (!strcmp(n, "IDP_ENA"))         { SCAN_INT(v, cf->idp_ena); break; }
+      if (!strcmp(n, "IMPS_ENA"))        { SCAN_INT(v, cf->imps_ena); break; }
       if (!strcmp(n, "IDP_PREF_ACS_BINDING")) { cf->idp_pref_acs_binding = v; break; }
       if (!strcmp(n, "IDPATOPT"))        { SCAN_INT(v, cf->idpatopt); break; }
+      if (!strcmp(n, "IDP_LIST_METH"))   { SCAN_INT(v, cf->idp_list_meth); break; }
       if (!strcmp(n, "INMAP"))           { cf->inmap = zxid_load_map(cf, cf->inmap, v); break; }
       goto badcf;
     case 'L':  /* LEVEL (log level) */
@@ -1077,13 +1112,9 @@ scan_end:
 	cf->path = v;
 	cf->path_len = strlen(v);
 	++cf->path_supplied;
-
-	buf = ZX_ALLOC(cf->ctx, ZXID_MAX_CONF);
-	len = read_all(ZXID_MAX_CONF-1, buf, "-conf_to_cf", "%szxid.conf", cf->path);
-	if (len > 0) {
-	  buf[len] = 0;
+	buf = read_all_alloc(cf->ctx, "-conf_to_cf", 1, &len, "%szxid.conf", cf->path);
+	if (buf && len)
 	  zxid_parse_conf_raw(cf, len, buf);  /* Recurse */
-	}
 	break;
       }
       if (!strcmp(n, "PDP_ENA"))        { SCAN_INT(v, cf->pdp_ena); break; }
@@ -1095,6 +1126,8 @@ scan_end:
       if (!strcmp(n, "PEPMAP_RSOUT"))   { cf->pepmap_rsout = zxid_load_map(cf, cf->pepmap_rsout, v); break; }
       if (!strcmp(n, "PEPMAP_RSIN"))    { cf->pepmap_rsin  = zxid_load_map(cf, cf->pepmap_rsin,  v); break; }
       if (!strcmp(n, "POST_A7N_ENC"))   { SCAN_INT(v, cf->post_a7n_enc); break; }
+      if (!strcmp(n, "POST_TEMPL_FILE"))   { cf->post_templ_file = v; break; }
+      if (!strcmp(n, "POST_TEMPL"))        { cf->post_templ = v; break; }
       goto badcf;
     case 'R':  /* RELY_A7N, RELY_MSG */
       if (!strcmp(n, "REDIRECT_HACK_IMPOSED_URL")) { cf->redirect_hack_imposed_url = v; break; }
@@ -1349,7 +1382,6 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "FEDUSERNAME_SUFFIX=%s\n"
 "#ZXID_CONF_FILE=%d (compile)\n"
 "#ZXID_CONF_FLAG=%d (compile)\n"
-"#ZXID_MAX_CONF=%d (compile)\n"
 "NON_STANDARD_ENTITYID=%s\n"
 "REDIRECT_HACK_IMPOSED_URL=%s\n"
 "REDIRECT_HACK_ZXID_URL=%s\n"
@@ -1376,6 +1408,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "WSPCGICMD=%s\n"
 "NAMEID_ENC=%x\n"
 "POST_A7N_ENC=%d\n"
+"CANON_INOPT=%x\n"
 "ENCKEY_OPT=%d\n"
 "IDPATOPT=%d\n"
 "DI_ALLOW_CREATE=%d\n"
@@ -1391,6 +1424,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "IPPORT=%s\n"
 "USER_LOCAL=%d\n"
 "IDP_ENA=%d\n"
+"IMPS_ENA=%d\n"
 "AS_ENA=%d\n"
 "PDP_ENA=%d\n"
 "#ZXID_MAX_BUF=%d (compile)\n"
@@ -1436,10 +1470,14 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "MOD_SAML_ATTR_PREFIX=%s\n"
 "BARE_URL_ENTITYID=%d\n"
 "SHOW_TECH=%d\n"
-"</pre>"
 
-"<textarea cols=100 rows=20>"
+"IDP_LIST_METH=%d\n"
 "IDP_SEL_PAGE=%s\n"
+"IDP_SEL_TEMPL_FILE=%s\n"
+"</pre>"
+"<textarea cols=100 rows=20>"
+"IDP_SEL_TEMPL=%s\n"
+#if 0
 "IDP_SEL_START=%s\n"
 "IDP_SEL_NEW_IDP=%s\n"
 "IDP_SEL_OUR_EID=%s\n"
@@ -1447,15 +1485,31 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "IDP_SEL_TECH_SITE=%s\n"
 "IDP_SEL_FOOTER=%s\n"
 "IDP_SEL_END=%s\n"
+#endif
+"</textarea><pre>\n"
 
 "AN_PAGE=%s\n"
 "AN_TEMPL_FILE=%s\n"
+"</pre><textarea cols=100 rows=20>"
 "AN_TEMPL=%s\n"
+"</textarea><pre>\n"
+
+"POST_TEMPL_FILE=%s\n"
+"</pre><textarea cols=100 rows=7>"
+"POST_TEMPL=%s\n"
+"</textarea><pre>\n"
+
+"ERR_PAGE=%s\n"
+"ERR_TEMPL_FILE=%s\n"
+"</pre><textarea cols=100 rows=7>"
+"ERR_TEMPL=%s\n"
+"</textarea><pre>\n"
 
 "NEW_USER_PAGE=%s\n"
 "RECOVER_PASSWD=%s\n"
 "ATSEL_PAGE=%s\n"
 
+"</pre><textarea cols=100 rows=15>"
 "MGMT_START=%s\n"
 "MGMT_LOGOUT=%s\n"
 "MGMT_DEFED=%s\n"
@@ -1463,7 +1517,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 "MGMT_END=%s\n"
 "</textarea>"
 
-"<pre>"
+"<pre>\n"
 "DBG=%s\n"
 
 "REQUIRED_AUTHN_CTX=\n%s\n"
@@ -1505,7 +1559,6 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 STRNULLCHK(cf->fedusername_suffix),
 		 ZXID_CONF_FILE,
 		 ZXID_CONF_FLAG,
-		 ZXID_MAX_CONF,
 		 STRNULLCHK(cf->non_standard_entityid),
 		 STRNULLCHK(cf->redirect_hack_imposed_url),
 		 STRNULLCHK(cf->redirect_hack_zxid_url),
@@ -1532,6 +1585,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 cf->wspcgicmd,
 		 cf->nameid_enc,
 		 cf->post_a7n_enc,
+		 cf->canon_inopt,
 		 cf->enckey_opt,
 		 cf->idpatopt,
 		 cf->di_allow_create,
@@ -1547,6 +1601,7 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 STRNULLCHK(cf->ipport),
 		 cf->user_local,
 		 cf->idp_ena,
+		 cf->imps_ena,
 		 cf->as_ena,
 		 cf->pdp_ena,
 		 ZXID_MAX_BUF,
@@ -1593,7 +1648,11 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 cf->bare_url_entityid,
 		 cf->show_tech,
 
+		 cf->idp_list_meth,
 		 STRNULLCHK(cf->idp_sel_page),
+		 STRNULLCHK(cf->idp_sel_templ_file),
+		 STRNULLCHK(cf->idp_sel_templ),
+#if 0
 		 STRNULLCHK(cf->idp_sel_start),
 		 STRNULLCHK(cf->idp_sel_new_idp),
 		 STRNULLCHK(cf->idp_sel_our_eid),
@@ -1601,11 +1660,18 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 		 STRNULLCHK(cf->idp_sel_tech_site),
 		 STRNULLCHK(cf->idp_sel_footer),
 		 STRNULLCHK(cf->idp_sel_end),
-
+#endif
 		 STRNULLCHK(cf->an_page),
 		 STRNULLCHK(cf->an_templ_file),
 		 STRNULLCHK(cf->an_templ),
 
+		 STRNULLCHK(cf->post_templ_file),
+		 STRNULLCHK(cf->post_templ),
+
+		 STRNULLCHK(cf->err_page),
+		 STRNULLCHK(cf->err_templ_file),
+		 STRNULLCHK(cf->err_templ),
+
 		 STRNULLCHK(cf->new_user_page),
 		 STRNULLCHK(cf->recover_passwd),
 		 STRNULLCHK(cf->atsel_page),
@@ -1640,5 +1706,4 @@ struct zx_str* zxid_show_conf(zxid_conf* cf)
 	 );
 }
 
-
 /* EOF  --  zxidconf.c */
