@@ -49,6 +49,7 @@ int zxid_sp_mni_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, struct zx_str*
     body = zx_NEW_e_Body(cf->ctx);
     body->ManageNameIDRequest = zxid_mk_mni(cf, zxid_get_user_nameid(cf, ses->nameid), new_nym, idp_meta);
     if (cf->sso_soap_sign) {
+      ZERO(&refs, sizeof(refs));
       refs.id = body->ManageNameIDRequest->ID;
       refs.canon = zx_EASY_ENC_SO_sp_ManageNameIDRequest(cf->ctx, body->ManageNameIDRequest);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert mni"))
@@ -122,7 +123,7 @@ struct zx_sp_ManageNameIDResponse_s* zxid_mni_do(zxid_conf* cf, zxid_cgi* cgi, z
   zxid_nid* nid;
   struct zx_str* newnym;
   
-  if (!zxid_chk_sig(cf, cgi, ses, (struct zx_elem_s*)mni, mni->Signature, mni->Issuer, "ManageNameIDRequest"))
+  if (!zxid_chk_sig(cf, cgi, ses, &mni->gg, mni->Signature, mni->Issuer, 0, "ManageNameIDRequest"))
     return 0;
   
   nid = zxid_decrypt_nameid(cf, mni->NameID, mni->EncryptedID);
