@@ -164,11 +164,12 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
   struct zx_root_s* r;
   struct zx_e_Header_s* hdr;
   struct zx_e_Body_s* body;
+  struct zx_wsse_Security_s* sec;
   struct zx_str* ss;
   struct zx_sp_Response_s* resp;
 
-#if 0
   hdr = zx_NEW_e_Header(cf->ctx);
+#if 0
   hdr->Action = zx_NEW_a_Action(cf->ctx);
   //hdr->Action->gg.content = zx_dup_str(cf->ctx, "urn:oasis:names:tc:xacml:2.0:profile:saml2.0:v2:schema:protocol:cd-01");
   hdr->Action->gg.content = zx_dup_str(cf->ctx, "urn:oasis:xacml:2.0:saml:protocol:schema:os");
@@ -177,18 +178,28 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
   //hdr->Action->gg.content = zx_dup_str(cf->ctx, "http://ws.apache.org/axis2/TestPolicyPortType/authRequestRequest");
   hdr->Action->actor = zx_ref_str(cf->ctx, SOAP_ACTOR_NEXT);
   hdr->Action->mustUnderstand = zx_ref_str(cf->ctx, ZXID_TRUE);
-#else
-  hdr = 0;
 #endif
 
+  /* Add our own token so PDP can do whatever PEP can (they are considered to be
+   * part of the same entity). This is TAS3 specific hack. */
+
+  sec = hdr->Security = zx_NEW_wsse_Security(cf->ctx);
+  sec->actor = zx_ref_str(cf->ctx, SOAP_ACTOR_NEXT);
+  sec->mustUnderstand = zx_ref_str(cf->ctx, ZXID_TRUE);
+  sec->Timestamp = zx_NEW_wsu_Timestamp(cf->ctx);
+  sec->Timestamp->Created = zx_NEW_wsu_Created(cf->ctx);
+  sec->Assertion = ses->tgta7n;
+  D("tgta7n=%p", ses->tgta7n);
+
   /* Prepare request according to the version */
 
   body = zx_NEW_e_Body(cf->ctx);
   if (!strcmp(cf->xasp_vers, "xac-soap")) {
-    body->xac_Request = zxid_mk_az_cd1(cf, subj, rsrc, act, env); /* *** warning: assignment from incompatible pointer type */
+    body->xac_Request = zxid_mk_xac_az(cf, subj, rsrc, act, env);
 #if 0
     /* *** xac:Response does not have signature field */
     if (cf->sso_soap_sign) {
+      ZERO(refs, sizeof(refs));
       refs.id = body->xac_Request->ID;
       refs.canon = zx_EASY_ENC_SO_xac_Request(cf->ctx, body->xac_Request);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az xac-soap"))
@@ -200,6 +211,7 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
   } else if (!strcmp(cf->xasp_vers, "2.0-cd1")) {
     body->xaspcd1_XACMLAuthzDecisionQuery = zxid_mk_az_cd1(cf, subj, rsrc, act, env);
     if (cf->sso_soap_sign) {
+      ZERO(&refs, sizeof(refs));
       refs.id = body->xaspcd1_XACMLAuthzDecisionQuery->ID;
       refs.canon = zx_EASY_ENC_SO_xaspcd1_XACMLAuthzDecisionQuery(cf->ctx, body->xaspcd1_XACMLAuthzDecisionQuery);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az cd1"))
@@ -210,6 +222,7 @@ static struct zx_sp_Response_s* zxid_az_soap(zxid_conf* cf, zxid_cgi* cgi, zxid_
   } else {
     body->XACMLAuthzDecisionQuery = zxid_mk_az(cf, subj, rsrc, act, env);
     if (cf->sso_soap_sign) {
+      ZERO(&refs, sizeof(refs));
       refs.id = body->XACMLAuthzDecisionQuery->ID;
       refs.canon = zx_EASY_ENC_SO_xasp_XACMLAuthzDecisionQuery(cf->ctx, body->XACMLAuthzDecisionQuery);
       if (zxid_lazy_load_sign_cert_and_pkey(cf, &sign_cert, &sign_pkey, "use sign cert az"))
@@ -471,7 +484,7 @@ char* zxid_az_cf_ses(zxid_conf* cf, const char* qs, zxid_ses* ses)
   char* ret;
   char* url = (cf->pdp_call_url&&*cf->pdp_call_url) ? cf->pdp_call_url : cf->pdp_url;
   D_INDENT("az: ");
-  memset(&cgi, 0 , sizeof(cgi));
+  ZERO(&cgi, sizeof(cgi));
   /*zxid_parse_cgi(&cgi, "");  DD("qs(%s) ses=%p", STRNULLCHKD(qs), ses);*/
   if (qs && ses)
     zxid_add_qs_to_ses(cf, ses, zx_dup_cstr(cf->ctx, qs), 1);
@@ -486,7 +499,7 @@ char* zxid_az_base_cf_ses(zxid_conf* cf, const char* qs, zxid_ses* ses)
   char* ret;
   char* url = (cf->pdp_call_url&&*cf->pdp_call_url) ? cf->pdp_call_url : cf->pdp_url;
   D_INDENT("azb: ");
-  memset(&cgi, 0 , sizeof(cgi));
+  ZERO(&cgi, sizeof(cgi));
   /*zxid_parse_cgi(&cgi, "");  DD("qs(%s) ses=%p", STRNULLCHKD(qs), ses);*/
   if (qs && ses)
     zxid_add_qs_to_ses(cf, ses, zx_dup_cstr(cf->ctx, qs), 1);
@@ -516,7 +529,7 @@ char* zxid_az_base_cf_ses(zxid_conf* cf, const char* qs, zxid_ses* ses)
 char* zxid_az_cf(zxid_conf* cf, const char* qs, const char* sid)
 {
   zxid_ses ses;
-  memset(&ses, 0 , sizeof(zxid_ses));
+  ZERO(&ses, sizeof(ses));
   if (sid && sid[0])
     zxid_get_ses(cf, &ses, sid);
   return zxid_az_cf_ses(cf, qs, &ses);
@@ -525,7 +538,7 @@ char* zxid_az_cf(zxid_conf* cf, const char* qs, const char* sid)
 char* zxid_az_base_cf(zxid_conf* cf, const char* qs, const char* sid)
 {
   zxid_ses ses;
-  memset(&ses, 0 , sizeof(zxid_ses));
+  ZERO(&ses, sizeof(ses));
   if (sid && sid[0])
     zxid_get_ses(cf, &ses, sid);
   return zxid_az_base_cf_ses(cf, qs, &ses);
@@ -540,7 +553,7 @@ char* zxid_az(const char* conf, const char* qs, const char* sid)
   struct zx_ctx ctx;
   zxid_conf cf;
   zx_reset_ctx(&ctx);
-  memset(&cf, 0, sizeof(zxid_conf));
+  ZERO(&cf, sizeof(cf));
   cf.ctx = &ctx;
   zxid_conf_to_cf_len(&cf, -1, conf);
   return zxid_az_cf(&cf, qs, sid);
@@ -551,7 +564,7 @@ char* zxid_az_base(const char* conf, const char* qs, const char* sid)
   struct zx_ctx ctx;
   zxid_conf cf;
   zx_reset_ctx(&ctx);
-  memset(&cf, 0, sizeof(zxid_conf));
+  ZERO(&cf, sizeof(cf));
   cf.ctx = &ctx;
   zxid_conf_to_cf_len(&cf, -1, conf);
   return zxid_az_base_cf(&cf, qs, sid);
