@@ -624,7 +624,6 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
   char* src;
   char* dst;
   char* lim;
-  int got;
   struct zx_str* issuer = 0;
   struct zx_str* affid;
   struct zx_str* nid;
@@ -633,12 +632,13 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
   struct zx_str* tgtnid;
   struct zx_str* accr;
   struct zx_str* path;
+  struct zx_sa_AuthnStatement_s* as;
   struct zx_sa_Assertion_s* a7n;
   struct zx_sa_Assertion_s* tgta7n;
-  char buf[ZXID_MAX_USER];
+  char* buf;
   char sha1_name[28];
 
-  D_INDENT("ses_to_pool: ");
+  D_INDENT("ses2pool: ");
   zxid_get_ses_sso_a7n(cf, ses);
   a7n = ses->a7n;
   D("adding a7n %p to pool", a7n);
@@ -660,9 +660,11 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
     zxid_user_sha1_name(cf, affid, nid, sha1_name);
     path = zx_strf(cf->ctx, "%s" ZXID_USER_DIR "%s", cf->path, sha1_name);
     zxid_add_attr_to_ses(cf, ses, "localpath",   path);
-    got = read_all(sizeof(buf)-1, buf, "splocal_user_at", "%.*s/.bs/.at", path->len, path->s);
-    if (got)
+    buf = read_all_alloc(cf->ctx, "splocal_user_at", 0, 0, "%.*s/.bs/.at", path->len, path->s);
+    if (buf) {
       zxid_add_ldif_attrs_to_ses(cf, ses, "local_", buf, "splocal_user_at");
+      ZX_FREE(cf->ctx, buf);
+    }
     zxid_copy_user_eprs_to_ses(cf, ses, path);
   }
 
@@ -690,18 +692,23 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
     zxid_user_sha1_name(cf, tgtaffid, tgtnid, sha1_name);
     path = zx_strf(cf->ctx, "%s" ZXID_USER_DIR "%s", cf->path, sha1_name);
     zxid_add_attr_to_ses(cf, ses, "tgtpath",   path);
-    got = read_all(sizeof(buf)-1, buf, "sptgt_user_at", "%.*s/.bs/.at", path->len, path->s);
-    if (got)
+    buf = read_all_alloc(cf->ctx, "sptgt_user_at", 0, 0, "%.*s/.bs/.at", path->len, path->s);
+    if (buf) {
       zxid_add_ldif_attrs_to_ses(cf, ses, "tgt_", buf, "sptgt_user_at");
+      ZX_FREE(cf->ctx, buf);
+    }
     zxid_copy_user_eprs_to_ses(cf, ses, path);
   }
   
-  accr = a7n&&a7n->AuthnStatement&&a7n->AuthnStatement->AuthnContext&&a7n->AuthnStatement->AuthnContext->AuthnContextClassRef&&a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content&&a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content?a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content:0;
+  accr = a7n&&(as = a7n->AuthnStatement)&&as->AuthnContext&&as->AuthnContext->AuthnContextClassRef?as->AuthnContext->AuthnContextClassRef->content:0;
+  //accr = a7n&&a7n->AuthnStatement&&a7n->AuthnStatement->AuthnContext&&a7n->AuthnStatement->AuthnContext->AuthnContextClassRef&&a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content&&a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content?a7n->AuthnStatement->AuthnContext->AuthnContextClassRef->content:0;
   zxid_add_attr_to_ses(cf, ses, "authnctxlevel", accr);
   
-  got = read_all(sizeof(buf)-1, buf, "splocal.all", "%s" ZXID_USER_DIR ".all/.bs/.at" , cf->path);
-  if (got)
+  buf = read_all_alloc(cf->ctx, "splocal.all", 0, 0, "%s" ZXID_USER_DIR ".all/.bs/.at" , cf->path);
+  if (buf) {
     zxid_add_ldif_attrs_to_ses(cf, ses, 0, buf, "splocal.all");
+    ZX_FREE(cf->ctx, buf);
+  }
   path = zx_strf(cf->ctx, "%s" ZXID_USER_DIR ".all", cf->path);
   zxid_copy_user_eprs_to_ses(cf, ses, path);
   
@@ -723,7 +730,7 @@ void zxid_ses_to_pool(zxid_conf* cf, zxid_ses* ses)
   URL_DECODE(dst, src, lim);
   *dst = 0;
   D("RelayState(%s)", ses->at->val);
-  D_DEDENT("ses_to_pool: ");
+  D_DEDENT("ses2pool: ");
 }
 
 /*(i) Add Attributes from Querty String to Session attribute pool
