@@ -15,6 +15,7 @@
  * 4.9.2009,  added attribute broker and PEP functionality --Sampo
  * 31.5.2010, moved local PEP and attribute broker functionality to zxidpep.c --Sampo
  * 7.9.2010,  tweaked the az requests to separate ses az from resource az --Sampo
+ * 22.9.2010, added People Service invitation resolution --Sampo
  *
  * Login button abbreviations
  * A2 = SAML 2.0 Artifact Profile
@@ -40,7 +41,7 @@
 #include "c/zxidvers.h"
 #include "c/zx-md-data.h"
 
-/*() Convert configuration string ~conf~ to configuration object ~cf~. */
+/*() Convert configuration string ~conf~ to configuration object ~cf~. See zxid_conf_to_cf() */
 
 /* Called by:  dirconf, main x2, zxid_az, zxid_fed_mgmt_len, zxid_idp_list_len, zxid_idp_select_len, zxid_new_conf_to_cf, zxid_simple_len */
 int zxid_conf_to_cf_len(zxid_conf* cf, int conf_len, const char* conf)
@@ -75,12 +76,9 @@ int zxid_conf_to_cf_len(zxid_conf* cf, int conf_len, const char* conf)
 
     if (!conf || conf_len < 5 || memcmp(conf, "PATH=", 5)) {
       /* No conf, or conf does not start by PATH: read from file default values */
-      buf = ZX_ALLOC(cf->ctx, ZXID_MAX_CONF);
-      len = read_all(ZXID_MAX_CONF-1, buf, "-conf_to_cf", "%szxid.conf", cf->path);
-      if (len > 0) {
-	buf[len] = 0;
+      buf = read_all_alloc(cf->ctx, "-conf_to_cf", 1, &len, "%szxid.conf", cf->path);
+      if (buf && len)
 	zxid_parse_conf_raw(cf, len, buf);
-      }
     }
     
     if (conf && conf_len) {
@@ -98,7 +96,12 @@ int zxid_conf_to_cf_len(zxid_conf* cf, int conf_len, const char* conf)
 /*() Create new ZXID configuration object given configuration string and
  * possibly configuration file.
  *
- * conf::   Configuration service
+ * zxid_new_conf_to_cf() parses first the default config file, then the string (i.e. string
+ * can override config file). However, if the string contains PATH specification,
+ * then the config file is reread from (presumably new) location and overrides
+ * eariler config.
+ *
+ * conf::   Configuration string
  * return:: Configuration object */
 
 /* Called by:  main x6, zxcall_main, zxidwspcgi_main x2 */
@@ -110,7 +113,7 @@ zxid_conf* zxid_new_conf_to_cf(const char* conf)
     ERR("out-of-memory %d", sizeof(zxid_conf));
     exit(1); /* *** perhaps too severe! */
   }
-  cf = memset(cf, 0, sizeof(zxid_conf));
+  cf = ZERO(cf, sizeof(zxid_conf));
   zxid_conf_to_cf_len(cf, -1, conf);
   return cf;
 }
@@ -239,12 +242,15 @@ char* zxid_fed_mgmt(char* conf, char* sid, int auto_flags) {
 /*() Bang-bang expansions (!!VAR) understood in the templates. */
 
 /* Called by:  zxid_template_page_cf */
-static char* zxid_map_bangbang(zxid_conf* cf, zxid_cgi* cgi, const char* key, const char* lim)
+static char* zxid_map_bangbang(zxid_conf* cf, zxid_cgi* cgi, const char* key, const char* lim, int auto_flags)
 {
   char* s;
   struct zx_str* ss;
   
   switch (*key) {
+  case 'A':
+    if (BBMATCH("ACTION_URL", key, lim)) return cgi->action_url;
+    break;
   case 'D':
     if (BBMATCH("DBG", key, lim)) return cgi->dbg;
     break;
@@ -256,16 +262,25 @@ static char* zxid_map_bangbang(zxid_conf* cf, zxid_cgi* cgi, const char* key, co
     }
     if (BBMATCH("ERR", key, lim)) return cgi->err;
     break;
+  case 'I':
+    if (BBMATCH("IDP_LIST", key, lim)) return zxid_idp_list_cf_cgi(cf, cgi, 0, auto_flags);
+    break;
   case 'M':
     if (BBMATCH("MSG", key, lim)) return cgi->msg;
     break;
   case 'U':
     if (BBMATCH("URL", key, lim)) return cf->url;
     break;
+  case 'R':
+    if (BBMATCH("RS", key, lim)) return cgi->rs;
+    break;
   case 'S':
+    if (BBMATCH("SIG", key, lim)) return cgi->sig;
     if (BBMATCH("SP_EID", key, lim)) return cgi->sp_eid;
     if (BBMATCH("SP_DPY_NAME", key, lim)) return cgi->sp_dpy_name;
     if (BBMATCH("SSOREQ", key, lim)) return cgi->ssoreq;
+    if (BBMATCH("SAML_ART", key, lim)) return cgi->saml_art;
+    if (BBMATCH("SAML_RESP", key, lim)) return cgi->saml_resp;
     break;
   case 'V':
     if (BBMATCH("VERSION", key, lim)) return zxid_version_str();
@@ -278,46 +293,52 @@ static char* zxid_map_bangbang(zxid_conf* cf, zxid_cgi* cgi, const char* key, co
   return 0;
 }
 
-/*() Expand a template. */
+/*() Expand a template. Only selected !!VAR expansions supported. No IFs or loops. */
 
 /* Called by:  zxid_simple_idp_show_an */
-static struct zx_str* zxid_template_page_cf(zxid_conf* cf, zxid_cgi* cgi, const char* templ_path, const char* default_templ)
+struct zx_str* zxid_template_page_cf(zxid_conf* cf, zxid_cgi* cgi, const char* templ_path, const char* default_templ, int size_hint, int auto_flags)
 {
   char buf[8192];
-  char* p;
-  char* q;
+  const char* templ;
+  const char* tp;
+  const char* tq;
   char* pp;
   struct zx_str* ss;
-  int len, got = read_all(sizeof(buf)-1, buf, "templ", "%s", templ_path);
+  int len, got = read_all(sizeof(buf)-1, buf, "templ", 1, "%s", templ_path);
   if (got <= 0) {
     D("Template at path(%s) not found. Using default template.", templ_path);
-    p = default_templ;
-  } else if (got <= 0 || got == sizeof(buf)-1) {
+    templ = default_templ;
+  } else if (got == sizeof(buf)-1) {
     ERR("Template at path(%s) does not fit in buffer of %d. Using default template.", templ_path, sizeof(buf)-1);
-    p = default_templ;
+    templ = default_templ;
   } else
-    p = buf;
-  ss = zx_new_len_str(cf->ctx, strlen(p) + 4096);
-  for (pp = ss->s; *p && pp < ss->s + ss->len; ) {
-    if (p[0] == '!' && p[1] == '!' && A_Z_a_z_(p[2])) {
-      for (q = p+=2; A_Z_a_z_(*p); ++p) ;
-      q = zxid_map_bangbang(cf, cgi, q, p);
-      if (!q || !*q)
-	continue;
-      len = strlen(q);
-      if (pp + len >= ss->s + ss->len) {
+    templ = buf;
+  while (1) {  /* Try rendering, iterate if expansion is needed. */
+    tp = templ;
+    ss = zx_new_len_str(cf->ctx, strlen(tp) + size_hint);
+    for (pp = ss->s; *tp && pp < ss->s + ss->len; ) {
+      if (tp[0] == '!' && tp[1] == '!' && A_Z_a_z_(tp[2])) {
+	for (tq = tp+=2; A_Z_a_z_(*tp); ++tp) ;
+	tq = zxid_map_bangbang(cf, cgi, tq, tp, auto_flags);
+	if (!tq || !*tq)
+	  continue;
+	len = strlen(tq);
+	if (pp + len >= ss->s + ss->len) {
+	  pp += len;
+	  break;
+	}
+	memcpy(pp, tq, len);
 	pp += len;
-	break;
+	continue;
       }
-      memcpy(pp, q, len);
-      pp += len;
+      *pp++ = *tp++;
+    }
+    if (pp >= ss->s + ss->len) {
+      INFO("Expansion of template too big. Does not fit in %d. Expanding.", ss->len);
+      size_hint += size_hint;  /* Double it */
       continue;
     }
-    *pp++ = *p++;
-  }
-  if (pp >= ss->s + ss->len) {
-    ERR("Expansion of template too big. Does not fit in %d", ss->len);
-    return 0;
+    break;
   }
   *pp = 0;
   return ss;
@@ -355,14 +376,13 @@ char* zxid_idp_list_cf_cgi(zxid_conf* cf, zxid_cgi* cgi, int* res_len, int auto_
   else
     ss = zx_dup_str(cf->ctx, "");
 
-#define ZXID_USE_POPUP
-#ifdef ZXID_USE_POPUP
-  dd = zx_strf(cf->ctx, "%.*s<select name=d>\n", ss->len, ss->s);
-  zx_str_free(cf->ctx, ss);
-  ss = dd;
-#endif
+  if (cf->idp_list_meth == ZXID_IDP_LIST_POPUP) {
+    dd = zx_strf(cf->ctx, "%.*s<select name=d>\n", ss->len, ss->s);
+    zx_str_free(cf->ctx, ss);
+    ss = dd;
+  }
 
-  D("Starting IdP processing... %p", idp);
+  D("Starting IdP list processing... %p", idp);
   for (; idp; idp = idp->n) {
     if (!idp->ed->IDPSSODescriptor)
       continue;
@@ -378,48 +398,55 @@ char* zxid_idp_list_cf_cgi(zxid_conf* cf, zxid_cgi* cgi, int* res_len, int auto_
       }
     }
 
-#ifdef ZXID_USE_POPUP
-    dd = zx_strf(cf->ctx, "%.*s"
-		 "<option value=\"%s\"> %s (%s) %s\n",
-		 ss->len, ss->s, idp->eid, STRNULLCHK(idp->dpy_name), idp->eid, mark);
-#else
-    if (cf->show_tech) {
+    switch (cf->idp_list_meth) {
+    default:
+      ERR("Unsupported IDP_LIST_METH=%d, reverting to popup.", cf->idp_list_meth);
+      cf->idp_list_meth = ZXID_IDP_LIST_POPUP;
+      /* fall thru */
+    case ZXID_IDP_LIST_POPUP:
       dd = zx_strf(cf->ctx, "%.*s"
-		   "<input type=submit name=\"l0%s\" value=\" Login to %s \">\n"
-		   "<input type=submit name=\"l1%s\" value=\" Login to %s (A2) \">\n"
-		   "<input type=submit name=\"l2%s\" value=\" Login to %s (P2) \">\n"
-		   "<input type=submit name=\"l5%s\" value=\" Login to %s (S2) \">%s<br>\n",
-		   ss->len, ss->s,
-		   idp->eid, idp->eid,
-		   idp->eid, idp->eid,
-		   idp->eid, idp->eid,
-		   idp->eid, idp->eid,
-		   mark);
-    } else {
-      dd = zx_strf(cf->ctx, "%.*s"
-		   "<input type=submit name=\"l0%s\" value=\" Login to %s \">%s<br>\n"
-		   ss->len, ss->s, idp->eid, idp->eid, mark);
+		   "<option value=\"%s\"> %s (%s) %s\n",
+		   ss->len, ss->s, idp->eid, STRNULLCHK(idp->dpy_name), idp->eid, mark);
+      break;
+    case ZXID_IDP_LIST_BUTTON:
+      if (cf->show_tech) {
+	dd = zx_strf(cf->ctx, "%.*s"
+		     "<input type=submit name=\"l0%s\" value=\" Login with %s (%s)\">\n"
+		     "<input type=submit name=\"l1%s\" value=\" Login with %s (%s) (A2) \">\n"
+		     "<input type=submit name=\"l2%s\" value=\" Login with %s (%s) (P2) \">\n"
+		     "<input type=submit name=\"l5%s\" value=\" Login with %s (%s) (S2) \">%s<br>\n",
+		     ss->len, ss->s,
+		     idp->eid, STRNULLCHK(idp->dpy_name), idp->eid,
+		     idp->eid, STRNULLCHK(idp->dpy_name), idp->eid,
+		     idp->eid, STRNULLCHK(idp->dpy_name), idp->eid,
+		     idp->eid, STRNULLCHK(idp->dpy_name), idp->eid,
+		     mark);
+      } else {
+	dd = zx_strf(cf->ctx, "%.*s"
+		     "<input type=submit name=\"l0%s\" value=\" Login with %s (%s) \">%s<br>\n",
+		     ss->len, ss->s, idp->eid, STRNULLCHK(idp->dpy_name), idp->eid, mark);
+      }
+      break;
     }
-#endif
     zx_str_free(cf->ctx, ss);
     ss = dd;
   }
-#ifdef ZXID_USE_POPUP
-  if (cf->show_tech) {
-    dd = zx_strf(cf->ctx, "%.*s</select>"
-		 "<input type=submit name=\"l0\" value=\" Login \">\n"
-		 "<input type=submit name=\"l1\" value=\" Login (A2) \">\n"
-		 "<input type=submit name=\"l2\" value=\" Login (P2) \">\n"
-		 "<input type=submit name=\"l5\" value=\" Login (S2) \"><br>\n",
-		 ss->len, ss->s);
-  } else {
-    dd = zx_strf(cf->ctx, "%.*s</select>"
-		 "<input type=submit name=\"l0\" value=\" Login \"><br>\n",
-		 ss->len, ss->s);
+  if (cf->idp_list_meth == ZXID_IDP_LIST_POPUP) {
+    if (cf->show_tech) {
+      dd = zx_strf(cf->ctx, "%.*s</select>"
+		   "<input type=submit name=\"l0\" value=\" Login \">\n"
+		   "<input type=submit name=\"l1\" value=\" Login (A2) \">\n"
+		   "<input type=submit name=\"l2\" value=\" Login (P2) \">\n"
+		   "<input type=submit name=\"l5\" value=\" Login (S2) \"><br>\n",
+		   ss->len, ss->s);
+    } else {
+      dd = zx_strf(cf->ctx, "%.*s</select>"
+		   "<input type=submit name=\"l0\" value=\" Login \"><br>\n",
+		   ss->len, ss->s);
+    }
+    zx_str_free(cf->ctx, ss);
+    ss = dd;
   }
-  zx_str_free(cf->ctx, ss);
-  ss = dd;
-#endif
 
   s = ss->s;
   D("IdP list(%s)", s);
@@ -459,16 +486,19 @@ struct zx_str* zxid_idp_select_zxstr_cf_cgi(zxid_conf* cf, zxid_cgi* cgi, int au
 {
   struct zx_str* eid=0;
   struct zx_str* ss;
-  char* idp_list;
 
-  if (cf->idp_sel_our_eid && cf->idp_sel_our_eid[0])
-    eid = zxid_my_entity_id(cf);
   D("HERE %p e(%s) m(%s) d(%s)", eid, FLDCHK(cgi, err), FLDCHK(cgi, msg), FLDCHK(cgi, dbg));
-  idp_list = zxid_idp_list_cf_cgi(cf, cgi, 0, auto_flags);
   if (cf->log_level>1)
     zxlog(cf, 0,0,0,0,0,0,0, "N", "W", "IDPSEL", 0, 0);
+
+#if 1
+  ss = zxid_template_page_cf(cf, cgi, cf->idp_sel_templ_file, cf->idp_sel_templ, 4096, auto_flags);
+#else
+  if (cf->idp_sel_our_eid && cf->idp_sel_our_eid[0])
+    eid = zxid_my_entity_id(cf);
+  char* idp_list = zxid_idp_list_cf_cgi(cf, cgi, 0, auto_flags);
   if ((auto_flags & ZXID_AUTO_FORMT) && (auto_flags & ZXID_AUTO_FORMF)) {
-    DD("HERE %p", idp_list);
+    DD("HERE %p", cgi->idp_list);
     ss = zx_strf(cf->ctx,
 		 "%s"
 #ifdef ZXID_USE_POST
@@ -530,6 +560,7 @@ struct zx_str* zxid_idp_select_zxstr_cf_cgi(zxid_conf* cf, zxid_cgi* cgi, int au
 		 FLDCHK(cgi, rs));
   } else
     ss = zx_dup_str(cf->ctx, "");
+#endif
 #if 0
   if (cgi.err) printf("<p><font color=red><i>%s</i></font></p>\n", cgi.err);
   if (cgi.msg) printf("<p><i>%s</i></p>\n", cgi.msg);
@@ -648,7 +679,7 @@ static char* zxid_simple_redir_page(zxid_conf* cf, char* redir, char* rs, int* r
  * N.B. More complete documentation is available in <<link: zxid-simple.pd>> (*** fixme) */
 
 /* Called by:  zxid_simple_no_ses_cf, zxid_simple_ses_active_cf x5 */
-static char* zxid_simple_show_idp_sel(zxid_conf* cf, zxid_cgi* cgi, int* res_len, int auto_flags)
+char* zxid_simple_show_idp_sel(zxid_conf* cf, zxid_cgi* cgi, int* res_len, int auto_flags)
 {
   struct zx_str* ss;
   D("cf=%p cgi=%p", cf, cgi);
@@ -694,7 +725,33 @@ static char* zxid_simple_show_conf(zxid_conf* cf, zxid_cgi* cgi, int* res_len, i
 {
   struct zx_str* ss = zxid_show_conf(cf);
   return zxid_simple_show_page(cf, ss, ZXID_AUTO_METAC, ZXID_AUTO_METAH,
-			       "c", "text/html", res_len, auto_flags);
+			       "d", "text/html", res_len, auto_flags);
+}
+
+/*() Show Error screen. */
+
+char* zxid_simple_show_err(zxid_conf* cf, zxid_cgi* cgi, int* res_len, int auto_flags)
+{
+  char* p;
+  struct zx_str* ss;
+
+  if (cf->log_level>1)
+    zxlog(cf, 0, 0, 0, 0, 0, 0, 0, "N", "W", "ERR", 0, "");
+
+  if (cf->err_page && cf->err_page[0]) {
+    ss = zx_strf(cf->ctx, "zxrfr=F%s%s%s%s&zxidpurl=%s",
+		 cgi->zxapp && cgi->zxapp[0] ? "&zxapp=" : "", cgi->zxapp ? cgi->zxapp : "",
+		 cgi->err && cgi->err[0] ? "&err=" : "", cgi->err ? cgi->err : "",
+		 cf->url);
+    p = ss->s;
+    ZX_FREE(cf->ctx, ss);
+    D("err_page(%s) p(%s)", cf->err_page, p);
+    return zxid_simple_redir_page(cf, cf->err_page, p, res_len, auto_flags);
+  }
+    
+  ss = zxid_template_page_cf(cf, cgi, cf->err_templ_file, cf->err_templ, 4096, auto_flags);
+  return zxid_simple_show_page(cf, ss, ZXID_AUTO_LOGINC, ZXID_AUTO_LOGINH,
+			       "g", "text/html", res_len, auto_flags);
 }
 
 /* ----------- IdP Screens ----------- */
@@ -775,7 +832,7 @@ static char* zxid_simple_idp_show_an(zxid_conf* cf, zxid_cgi* cgi, int* res_len,
   struct zx_root_s* r;
   struct zx_str* ss;
   zxid_ses sess;
-  memset(&sess, 0 , sizeof(sess));
+  ZERO(&sess, sizeof(sess));
   D("cf=%p cgi=%p", cf, cgi);
   
   DD("z saml_req(%s) rs(%s) sigalg(%s) sig(%s)", cgi->saml_req, cgi->rs, cgi->sigalg, cgi->sig);  
@@ -852,7 +909,7 @@ static char* zxid_simple_idp_show_an(zxid_conf* cf, zxid_cgi* cgi, int* res_len,
       cgi->sp_dpy_name = "--No SP could be determined--";
   }
   
-  ss = zxid_template_page_cf(cf, cgi, cf->an_templ_file, cf->an_templ);
+  ss = zxid_template_page_cf(cf, cgi, cf->an_templ_file, cf->an_templ, 4096, auto_flags);
   if (b64)
     ZX_FREE(cf->ctx, b64);
   DD("an_page: ret(%s)", ss?ss->len:1, ss?ss->s:"?");
@@ -875,7 +932,8 @@ static char* zxid_simple_idp_pw_authn(zxid_conf* cf, zxid_cgi* cgi, int* res_len
  
   if (!zxid_decode_ssoreq(cf, cgi))
     goto err;
-  
+
+  ZERO(&sess, sizeof(sess));
   if (zxid_pw_authn(cf, cgi, &sess))
     return zxid_simple_idp_an_ok_do_rest(cf, cgi, &sess, res_len, auto_flags);
 
@@ -988,12 +1046,14 @@ char* zxid_simple_ses_active_cf(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, int
    * A = Artifact processing
    * N = New User, during IdP Login (form an)
    * W = Recover password,  during IdP Login (form aw)
+   * D = Delegation / Invitation acceptance user interface, the idp selection
+   * G = Delegation / Invitation finalization after SSO (via RelayState)
    *
    * I = used for IdP ???
    * K = used?
    * F = IdP: Return SSO A7N after successful An; no ses case, generate IdP ui
    *
-   * Still available: DGHJORTUVWXYZabcdefghijkoqwxyz
+   * Still available: GHJORTUVWXYZabcdefghijkoqwxyz
    */
   
   if (cgi->enc_hint)
@@ -1045,6 +1105,11 @@ char* zxid_simple_ses_active_cf(zxid_conf* cf, zxid_cgi* cgi, zxid_ses* ses, int
     }
     D("Q ss(%.*s) (fall thru)", ss->len, ss->s);
     break;
+
+     /*  Delegation / Invitation URL clicked. */
+  case 'D':  return zxid_ps_accept_invite(cf, cgi, ses, res_len, auto_flags);
+  case 'G':  return zxid_ps_finalize_invite(cf, cgi, ses, res_len, auto_flags);
+
   case 'R':
     cgi->op = 'F';
     /* Fall thru */
@@ -1219,6 +1284,8 @@ show_protected_content_setcookie:
   case 'c':    return zxid_simple_show_carml(cf, cgi, res_len, auto_flags);
   case 'd':    return zxid_simple_show_conf(cf, cgi, res_len, auto_flags);
   case 'B':    return zxid_simple_show_meta(cf, cgi, res_len, auto_flags);
+  case 'D': /*  Delegation / Inviatation URL clicked. */
+     return zxid_ps_accept_invite(cf, cgi, ses, res_len, auto_flags);
   case 'R':
     cgi->op = 'F';
     /* Fall thru */
@@ -1281,7 +1348,7 @@ char* zxid_simple_cf_ses(zxid_conf* cf, int qs_len, char* qs, zxid_ses* ses, int
   char* buf = 0;
   char* res = 0;
   zxid_cgi cgi;
-  memset(&cgi, 0, sizeof(cgi));
+  ZERO(&cgi, sizeof(cgi));
   
   if (!cf || !ses) {
     ERR("NULL pointer. You MUST supply configuration object %p and session object %p (programming error). auto_flags=%x", cf, ses, auto_flags);
@@ -1375,7 +1442,7 @@ char* zxid_simple_cf_ses(zxid_conf* cf, int qs_len, char* qs, zxid_ses* ses, int
 	  goto done;
   }
 
-  memset(ses, 0, sizeof(ses));   /* No session yet! Process login form */
+  ZERO(ses, sizeof(ses));   /* No session yet! Process login form */
   res = zxid_simple_no_ses_cf(cf, &cgi, ses, res_len, auto_flags);
 
 done:
@@ -1401,7 +1468,7 @@ done:
 char* zxid_simple_cf(zxid_conf* cf, int qs_len, char* qs, int* res_len, int auto_flags)
 {
   zxid_ses ses;
-  memset(&ses, 0, sizeof(ses));
+  ZERO(&ses, sizeof(ses));
   return zxid_simple_cf_ses(cf, qs_len, qs, &ses, res_len, auto_flags);
 }
 
@@ -1417,7 +1484,7 @@ char* zxid_simple_len(int conf_len, char* conf, int qs_len, char* qs, int* res_l
   struct zx_ctx ctx;
   zxid_conf cf;
   zx_reset_ctx(&ctx);
-  memset(&cf, 0, sizeof(zxid_conf));
+  ZERO(&cf, sizeof(cf));
   cf.ctx = &ctx;
   zxid_conf_to_cf_len(&cf, conf_len, conf);
   return zxid_simple_cf(&cf, qs_len, qs, res_len, auto_flags);
