@@ -116,7 +116,7 @@ int zxid_map_sec_mech(zxid_epr* epr)
 /* Called by:  zxid_wsf_sign */
 int zxid_hunt_sig_parts(zxid_conf* cf, int n_refs, struct zxsig_ref* refs, struct zx_ds_Reference_s* sref, struct zx_e_Header_s* hdr, struct zx_e_Body_s* bdy)
 {
-  for (; sref && n_refs < ZXID_N_WSF_SIGNED_HEADERS; sref = ZX_NEXT(sref)) {
+  for (; sref && n_refs < ZXID_N_WSF_SIGNED_HEADERS; sref = (void*)ZX_NEXT(sref)) {
     if (!sref->URI || !sref->URI->len || !sref->URI->s || !sref->URI->s[0]) {
       ERR("Malformed signature: Reference is missing URI %p n_refs=%d", sref->URI, n_refs);
       continue;
