@@ -24,9 +24,11 @@
 #include <stdlib.h>
 #include <errno.h>
 #include <sys/types.h>
-#include <sys/stat.h>
 #include <signal.h>
 #include <fcntl.h>
+#ifndef MINGW
+#include <sys/stat.h>
+#endif
 
 #ifdef USE_OPENSSL
 #include <openssl/des.h>
@@ -42,7 +44,7 @@
 
 #define UDIR "/var/zxid/idpuid"
 
-CU8* help =
+char* help =
 "zxpasswd  -  Password creation and user management tool R" ZXID_REL "\n\
 Copyright (c) 2009-2010 Sampo Kellomaki (sampo@iki.fi), All Rights Reserved.\n\
 NO WARRANTY, not even implied warranties. Licensed under Apache License v2.0\n\
@@ -52,9 +54,10 @@ Send well researched bug reports to the author. Home: zxid.org\n\
 Usage: zxpasswd [options] user [udir] <passwd    # Set user's password\n\
        zxpasswd [options] -c user [udir] <passwd # Create user and set password\n\
        zxpasswd [options] -a user [udir] <passwd # Authenticate as user using pw\n\
-       zxpasswd [options] -l [user] [udir]       # List information about user\n\
+       zxpasswd [options] -l [user [udir]]       # List information about user\n\
   [udir]           Specify zxididp user directory. Default " UDIR "\n\
   -c               Create user\n\
+  -at 'attr: val'  Append attribute(s) to .bs/.at\n\
   -s exist_uid     Symlink user to an existing user (e.g. yubikey alias)\n\
   -a               Authenticate as user. exit(2) value 0 means success\n\
   -l               List user info. If no user is specified, lists all users.\n\
@@ -69,22 +72,19 @@ For Yubikey (yubico.com) authentication (-a), supply the yubikey ticket\n\
 as user and omit the password. For creating account or changing password,\n\
 use -t y to indicate that you pass yubikey AES128 shared key in hex as password.\n";
 
-char zx_instance[64] = "\tzxpw";
-int zx_debug = 0;
-char zx_indent[256] = "";
 int verbose = 1;
-int assert_nonfatal = 0;
-char* assert_msg = "assert fired.";
 int create = 0;
 int an = 0;
 int list = 0;
 char* hash_type = "1";
-char* udir = 0;
+char* udir = "/var/zxid/idpuid/";
 char* user = 0;
 char* symlink_user = 0;
+char* at = 0;
 char pw[1024];
 char userdir[4096];
 char buf[4096];
+struct zx_ctx ctx;
 
 /* Called by:  main x8, zxcall_main, zxcot_main */
 static void opt(int* argc, char*** argv, char*** env)
@@ -118,6 +118,11 @@ static void opt(int* argc, char*** argv, char*** env)
       case '\0':
 	++an;
 	continue;
+      case 't':
+	++(*argv); --(*argc);
+	if ((*argc) < 1) break;
+	at = (*argv)[0];
+	continue;
       }
       break;
 
@@ -192,7 +197,7 @@ static void opt(int* argc, char*** argv, char*** env)
     exit(3);
   }
  last:
-  if (!*argc) {
+  if (!list && !*argc) {
     fprintf(stderr, "Too few arguments (%d). Must specify at least user name.\n", *argc);
     goto help;
   }
@@ -202,31 +207,31 @@ static void opt(int* argc, char*** argv, char*** env)
 static int list_user(char* userdir, char* udir)
 {
   int got;
+  char* at;
   struct dirent* de;
   DIR* dir;
-
-  printf("User dir:              %s\n", userdir);
-  got = read_all(sizeof(buf), buf, "pw", "%s/%s/.pw", udir, user);
-  printf("Password hash:         %s\n", buf);
-  got = read_all(sizeof(buf), buf, "at", "%s/%s/.bs/.at", udir, user);
-  printf("User attributes:       %s\n", buf);
-  got = read_all(sizeof(buf), buf, "all at", "%s/.all/.bs/.at", udir, 0);
-  printf("All user's attributes: %s\n", buf);
-
-  printf("User's Federated SPs\n");
-
   dir = opendir(userdir);
   if (!dir) {
     perror("opendir for /var/zxid/idpuid/USER userdir (or other if configured)");
     D("failed path(%s)", userdir);
-    return 0;
+    return 4;
   }
+  printf("User dir:              %s\n", userdir);
+  got = read_all(sizeof(buf), buf, "pw", 0, "%s/%s/.pw", udir, user);
+  printf("Password hash:         %s\n", buf);
+  at = read_all_alloc(&ctx, "at", 0, 0, "%s/%s/.bs/.at", udir, user);
+  if (at) printf("User attributes:       %s\n", at);
+  at = read_all_alloc(&ctx, "all at", 0, 0, "%s/.all/.bs/.at", udir, 0);
+  if (at) printf("Common (.all) user attributes: %s\n", buf);
+
+  printf("User's Federated SPs\n");
+
   while (de = readdir(dir))
     if (de->d_name[0] != '.' && de->d_name[strlen(de->d_name)-1] != '~') {
-      got = read_all(sizeof(buf), buf, "sp at", "%s/%s/.mni", userdir, de->d_name);
+      got = read_all(sizeof(buf), buf, "sp at", 0, "%s/%s/.mni", userdir, de->d_name);
       printf("SP specific NameID:  %s (%s)\n", buf, de->d_name);
-      got = read_all(sizeof(buf), buf, "sp at", "%s/%s/.at", userdir, de->d_name);
-      printf("SP specific attrib:  %s (%s)\n", buf, de->d_name);
+      at = read_all_alloc(&ctx, "sp at", 0, 0, "%s/%s/.at", userdir, de->d_name);
+      if (at) printf("SP specific attrib:  %s (%s)\n", buf, de->d_name);
     }
 
   /* *** TODO: .all SPs, bootstraps, discovery regs */
@@ -241,6 +246,7 @@ static int list_user(char* userdir, char* udir)
 static int list_users(char* udir)
 {
   int got;
+  char* at;
   struct dirent* de;
   DIR* dir;
 
@@ -248,14 +254,14 @@ static int list_users(char* udir)
   if (!dir) {
     perror("opendir for /var/zxid/idpuid (or other if configured)");
     D("failed path(%s)", udir);
-    return 0;
+    return 1;
   }
   while (de = readdir(dir))
     if (de->d_name[0] != '.' && de->d_name[strlen(de->d_name)-1] != '~') {
-      got = read_all(sizeof(buf), buf, "sp at", "%s/%s/.mni", userdir, de->d_name);
+      got = read_all(sizeof(buf), buf, "sp at", 0, "%s/%s/.mni", userdir, de->d_name);
       printf("SP specific NameID:  %s (%s)\n", buf, de->d_name);
-      got = read_all(sizeof(buf), buf, "sp at", "%s/%s/.at", userdir, de->d_name);
-      printf("SP specific attrib:  %s (%s)\n", buf, de->d_name);
+      at = read_all_alloc(&ctx, "sp at", 0, 0, "%s/%s/.bs/.at", userdir, de->d_name);
+      if (at) printf("SP specific attrib:  %s (%s)\n", buf, de->d_name);
     }
   
   closedir(dir);
@@ -269,11 +275,14 @@ int main(int argc, char** argv, char** env)
 {
   int isyk = 0;
   int got, pwgot;
+  char* p;
   unsigned char salt[16];
   unsigned char pw_hash[120];
   unsigned char ch;
   yubikey_token_st yktok;
   
+  strcpy(zx_instance, "\tzxpw");
+  zx_reset_ctx(&ctx);
   opt(&argc, &argv, &env);
   if (argc)
     user = argv[0];
@@ -290,7 +299,7 @@ int main(int argc, char** argv, char** env)
     userdir[sizeof(userdir)-1] = 0;
   }
   if (list) {
-    if (user)
+    if (user && user[0])  /* passing empty user results full listing */
       return list_user(userdir, udir);
     else
       return list_users(udir);
@@ -307,8 +316,8 @@ int main(int argc, char** argv, char** env)
     read_all_fd(0, pw, sizeof(pw)-1, &pwgot);  /* Password from stdin */
   }
   if (pwgot) {
-    if (pw[pwgot-1] == '\015') --pwgot;
     if (pw[pwgot-1] == '\012') --pwgot;
+    if (pw[pwgot-1] == '\015') --pwgot;
   }
   pw[pwgot] = 0;
   D("pw(%s) len=%d", pw, pwgot);
@@ -319,19 +328,19 @@ int main(int argc, char** argv, char** env)
     if (isyk) {
       snprintf(userdir, sizeof(userdir)-1, "%s/%s", udir, user);
       userdir[sizeof(userdir)-1] = 0;
-      got = read_all(sizeof(buf), buf, "ykspent", "%s/.ykspent/%s", userdir, pw);
+      got = read_all(sizeof(buf), buf, "ykspent", 1, "%s/.ykspent/%s", userdir, pw);
       if (got) {
 	ERR("The One Time Password has already been spent. ticket(%s%s) buf(%.*s)", user, pw, got, buf);
-	return 1;
+	return 5;
       }
       if (!write_all_path_fmt("ykspent", sizeof(buf), buf, "%s/.ykspent/%s", userdir, pw, "1"))
 	return 1;
 
-      got = read_all(sizeof(buf), buf, "ykaes", "%s/%s/.yk", udir, user);
+      got = read_all(sizeof(buf), buf, "ykaes", 1, "%s/%s/.yk", udir, user);
       D("buf    (%s) got=%d", buf, got);
       if (got < 32) {
 	ERR("User's %s/.yk file must contain aes128 key as 32 hexadecimal characters. Too few characters %d ticket(%s)", user, got, pw);
-	return 1;
+	return 6;
       }
       if (got > 32) {
 	INFO("User's %s/.yk file must contain aes128 key as 32 hexadecimal characters. Too many characters %d ticket(%s). Truncating.", user, got, pw);
@@ -339,9 +348,9 @@ int main(int argc, char** argv, char** env)
 	buf[got] = 0;
       }
       zx_hexdec(buf, buf, got, hex_trans);
-      memset (&yktok, 0, sizeof(yktok));
+      ZERO(&yktok, sizeof(yktok));
       zx_hexdec((void *)&yktok, pw, pwgot, ykmodhex_trans);
-      yubikey_aes_decrypt((void *)&yktok, buf);
+      yubikey_aes_decrypt((void *)&yktok, (unsigned char*)buf);
       D("internal uid %02x %02x %02x %02x %02x %02x counter=%d 0x%x timestamp=%d (hi=%x lo=%x) use=%d 0x%x rnd=0x%x crc=0x%x", yktok.uid[0], yktok.uid[1], yktok.uid[2], yktok.uid[3], yktok.uid[4], yktok.uid[5], yktok.ctr, yktok.ctr, (yktok.tstph << 16) | yktok.tstpl, yktok.tstph, yktok.tstpl, yktok.use, yktok.use, yktok.rnd, yktok.crc);
             
       if (yubikey_crc_ok_p((unsigned char*)&yktok)) {
@@ -349,34 +358,51 @@ int main(int argc, char** argv, char** env)
 	return 0;
       }
       D("yubikey ticket validation failure %d", 0);
-      return 1;
+      return 7;
     }
-    got = read_all(sizeof(buf), buf, "pw", "%s/%s/.pw", udir, user);
-    if (buf[got-1] == '\n') buf[got--] = 0;
-    if (buf[got-1] == '\r') buf[got--] = 0;
-    D("buf    (%s)", buf);
+    got = read_all(sizeof(buf), buf, "pw", 1, "%s/%s/.pw", udir, user);
+    if (got>0) {
+      if (buf[got-1] == '\012') --got;
+      if (buf[got-1] == '\015') --got;
+    }
+    buf[got] = 0;
+    D("buf    (%s) got=%d", buf, got);
     if (!memcmp(buf, "$1$", sizeof("$1$")-1)) {
-      zx_md5_crypt(pw, buf, pw_hash);
+      zx_md5_crypt(pw, buf, (char*)pw_hash);
       D("pw_hash(%s)", pw_hash);
-      return strcmp(buf, pw_hash)?1:0;
+      return strcmp(buf, (char*)pw_hash)?7:0;
     }
 #ifdef USE_OPENSSL
     if (!memcmp(buf, "$c$", sizeof("$c$")-1)) {
-      DES_fcrypt(pw, buf+3, pw_hash);
+      DES_fcrypt(pw, buf+3, (char*)pw_hash);
       D("pw_hash(%s)", pw_hash);
-      return strcmp(buf+3, pw_hash)?1:0;
+      return strcmp(buf+3, (char*)pw_hash)?7:0;
     }
 #endif
     if (ONE_OF_2(buf[0], '$', '_')) {
       fprintf(stderr, "Unsupported password hash algorithm (%s).\n", buf);
-      return 1;
+      return 8;
     }
     D("Assume plain text password %d", 0);
-    return strcmp(buf, pw)?1:0;
+    return strcmp(buf, pw)?7:0;
   }
+
+  /* Create and other user management functions */
   
   if (create) {
-    MKDIR(userdir, 0770);
+    if (MKDIR(userdir, 0770) == -1) {
+      ERR("User already exists %s", userdir);
+      return 3;
+    }
+    snprintf(buf, sizeof(buf)-1, "%s/.bs", userdir);
+    buf[sizeof(buf)-1] = 0; /* must terminate manually as on win32 nul is not guaranteed */
+    MKDIR(buf, 0770);
+    snprintf(buf, sizeof(buf)-1, "%s/.ps", userdir);
+    buf[sizeof(buf)-1] = 0; /* must terminate manually as on win32 nul is not guaranteed */
+    MKDIR(buf, 0770);
+    snprintf(buf, sizeof(buf)-1, "%s/.ykspent", userdir);
+    buf[sizeof(buf)-1] = 0; /* must terminate manually as on win32 nul is not guaranteed */
+    MKDIR(buf, 0770);
   }
   if (symlink_user) {
     snprintf(buf, sizeof(buf), "%s/%s", udir, symlink_user);
@@ -392,27 +418,37 @@ int main(int argc, char** argv, char** env)
 #endif
   }
   
+  if (at) {
+    for (p = at; *p; ++p)
+      if (*p == '$') *p = '\n';
+    snprintf(buf, sizeof(buf)-1, "%s/.bs/.at", userdir);
+    D("Appending to(%s) attributes(%s)", buf, at);
+    buf[sizeof(buf)-1] = 0; /* must terminate manually as on win32 nul is not guaranteed */
+    write2_or_append_lock_c_path(buf, strlen(at), at, 0, 0, "append .bs/.at", SEEK_END, O_APPEND);
+  }
+  
   /* $1$$6C2jXXYmjnyAkfWXmnCSf0 */
   /* $y$$6012cab434c66ab87d43d4babe463331 */
 
   if (!strcmp(hash_type, "0")) {
-    strcpy(pw_hash, pw);
+    strcpy((char*)pw_hash, pw);
+    D("pw0(%s) len=%d", pw, strlen(pw));
 #ifdef USE_OPENSSL
   } else if (!strcmp(hash_type, "c")) {  /* Unix crypt(3) hash */
-    zx_rand(salt, 2);
+    zx_rand((char*)salt, 2);
     salt[0] = pw_basis_64[salt[0] & 0x3f];
     salt[1] = pw_basis_64[salt[1] & 0x3f];
-    strcpy(pw_hash, "$c$");  /* Our custom magic to identify Unix crypt(3) hash */
-    DES_fcrypt(pw, salt, pw_hash+3);
+    strcpy((char*)pw_hash, "$c$");  /* Our custom magic to identify Unix crypt(3) hash */
+    DES_fcrypt(pw, (char*)salt, (char*)pw_hash+3);
 #endif
   } else if (!strcmp(hash_type, "1")) {  /* MD5 hash */
     for (got = 0; got < 8; ++got) {
-      zx_rand(&ch, 1);
+      zx_rand((char*)&ch, 1);
       salt[got] = pw_basis_64[ch & 0x3f];
     }
     salt[8] = 0;
     D("salt(%s)", salt);
-    zx_md5_crypt(pw, salt, pw_hash);
+    zx_md5_crypt(pw, (char*)salt, (char*)pw_hash);
     D("pw_hash(%s)", pw_hash);
   } else if (!strcmp(hash_type, "y")) {
     D("Provisioning yubikey aes(%s) in %s/%s/.yk", pw, udir, user);
@@ -426,6 +462,7 @@ int main(int argc, char** argv, char** env)
     fprintf(stderr, "Unsupported password hash algorithm (%s).\n", hash_type);
   }
   
+  DD("pw_hash(%s) len=%d", pw_hash, strlen(pw_hash));
   if (!write_all_path_fmt("set pw", sizeof(buf), buf, "%s/%s/.pw", udir, user, "%s", pw_hash))
     return 1;
   return 0;
